#define RELEASE_STRING 9.3.0
#define MAJOR_RELEASE_STRING 9.3
#define BUILD_NUMBER 9.3.0.0089
#define MAJOR_VERSION 9
#define MINOR_VERSION 3
#define PATCH_VERSION 0
#define BUILD_SERIAL_NUMBER 0089
#define VERSION_STRING "9.3.0.0089"
#define PRODUCT_STRING "9.3"
#define PACKAGE_STRING "CUBRID 9.3"

