LIBRARY   cascci
;EXETYPE   WINDOWS
;CODE      PRELOAD MOVEABLE DISCARDABLE
;DATA      PRELOAD MOVEABLE SINGLE
HEAPSIZE  1024
EXPORTS
	cci_init
	cci_end
	cci_get_version
	cci_connect_3_0
	cci_connect_ex
	cci_connect_with_url
	cci_connect_with_url_ex
	cci_disconnect
	cci_end_tran
	cci_prepare
	cci_get_bind_num
	cci_get_result_info
	cci_bind_param
	cci_execute
	cci_prepare_and_execute
	cci_get_db_parameter
	cci_set_db_parameter
	cci_close_query_result
	cci_close_req_handle
	cci_cursor
	cci_fetch_size
	cci_fetch
	cci_get_data
	cci_schema_info
	cci_get_cur_oid
	cci_oid_get
	cci_oid_put
	cci_oid_put2
	cci_get_db_version
	cci_get_class_num_objs
	cci_oid
	cci_oid_get_class_name
	cci_col_get
	cci_col_size
	cci_col_set_drop
	cci_col_set_add
	cci_col_seq_drop
	cci_col_seq_insert
	cci_col_seq_put
	cci_is_updatable
	cci_is_holdable
	cci_next_result
	cci_bind_param_array_size
	cci_bind_param_array
	cci_execute_array
	cci_query_result_free
	cci_fetch_sensitive
	cci_cursor_update
	cci_execute_batch
	cci_fetch_buffer_clear
	cci_execute_result
	cci_set_isolation_level
	cci_set_lock_timeout
	cci_set_free
	cci_set_size
	cci_set_element_type
	cci_set_get
	cci_set_make
	cci_set_query_timeout
	cci_get_attr_type_str
	cci_get_query_plan
	cci_get_query_timeout
	cci_query_info_free
	cci_set_max_row
	cci_savepoint
	cci_get_param_info
	cci_param_info_free
	cci_blob_new
	cci_blob_size
	cci_blob_write
	cci_blob_read
	cci_blob_free
	cci_clob_new
	cci_clob_size
	cci_clob_write
	cci_clob_read
	cci_clob_free
	cci_get_dbms_type
	cci_register_out_param
	cci_cancel
	cci_get_error_msg
	cci_get_err_msg
	
	cci_set_charset
	cci_row_count
	cci_get_shard_id_with_req_handle
	cci_last_insert_id
	cci_get_last_insert_id
	cci_get_autocommit
	cci_set_autocommit
	cci_property_create
	cci_property_destroy
	cci_property_set
	cci_property_get
	cci_datasource_create
	cci_datasource_destroy
	cci_datasource_borrow
	cci_datasource_release
	cci_set_allocators
	cci_escape_string
	cci_set_holdability
	cci_get_holdability
	
	cci_log_get
	cci_log_finalize
	cci_log_write
	cci_log_writef
	cci_log_remove
	cci_log_set_level
	cci_log_is_writable
	cci_log_set_force_flush
	cci_log_set_default_postfix
	
;          WEP PRIVATE
; To implement your own Windows Exit Procedure add the following
; function to your application (referring to it in the .def file is
; not required.)  The extern "C" is only required if module is C++.
; extern "C" int FAR PASCAL _WEP(int)
; {
;       /* Your WEP functionality goes here */
;  return 1;
; }

